@@ -6,7 +6,7 @@
  * You may not use this file except in compliance with the License.
  *
  * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
- * or http://www.opensolaris.org/os/licensing.
+ * or https://opensource.org/licenses/CDDL-1.0.
  * See the License for the specific language governing permissions
  * and limitations under the License.
  *
@@ -35,8 +35,8 @@ extern "C" {
 #define	KM_SLEEP	0x00000000	/* same as KM_SLEEP */
 #define	KM_NOSLEEP	0x00000001	/* same as KM_NOSLEEP */
 
-#define	kmem_alloc(size, flags)		malloc(size)
-#define	kmem_free(ptr, size)		free(ptr)
+#define	kmem_alloc(size, flags)		((void) sizeof (flags), malloc(size))
+#define	kmem_free(ptr, size)		((void) sizeof (size), free(ptr))
 
 #ifdef	__cplusplus
 }
